package admin_promotion

import (
	"context"
	"fmt"
	"time"

	"tyc-server/app/main/api/internal/svc"
	"tyc-server/app/main/api/internal/types"
	"tyc-server/app/main/model"
	"tyc-server/common/xerr"

	"github.com/pkg/errors"
	"github.com/zeromicro/go-zero/core/logx"
	"github.com/zeromicro/go-zero/core/stores/sqlx"
)

type RecordLinkClickLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewRecordLinkClickLogic(ctx context.Context, svcCtx *svc.ServiceContext) *RecordLinkClickLogic {
	return &RecordLinkClickLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

func (l *RecordLinkClickLogic) RecordLinkClick(req *types.RecordLinkClickReq) (resp *types.RecordLinkClickResp, err error) {
	// 校验路径格式
	if len(req.Path) != 6 {
		return nil, errors.Wrapf(xerr.NewErrCode(xerr.SERVER_COMMON_ERROR), "无效的推广链接路径")
	}

	// 检查是否只包含大小写字母和数字
	for _, char := range req.Path {
		if !((char >= 'a' && char <= 'z') || (char >= 'A' && char <= 'Z') || (char >= '0' && char <= '9')) {
			return nil, errors.Wrapf(xerr.NewErrCode(xerr.SERVER_COMMON_ERROR), "无效的推广链接路径")
		}
	}
	url := fmt.Sprintf("%s/%s", l.svcCtx.Config.AdminPromotion.URLDomain, req.Path)

	link, err := l.svcCtx.AdminPromotionLinkModel.FindOneByUrl(l.ctx, url)
	if err != nil {
		return nil, errors.Wrapf(xerr.NewErrCode(xerr.DB_ERROR), "无效的推广链接路径")
	}

	err = l.svcCtx.AdminPromotionLinkModel.Trans(l.ctx, func(ctx context.Context, session sqlx.Session) error {
		// 更新总统计
		totalStats, err := l.svcCtx.AdminPromotionLinkStatsTotalModel.FindOneByLinkId(l.ctx, link.Id)
		if err != nil {
			return err
		}
		totalStats.ClickCount = totalStats.ClickCount + 1
		err = l.svcCtx.AdminPromotionLinkStatsTotalModel.UpdateWithVersion(l.ctx, session, totalStats)
		if err != nil {
			return fmt.Errorf("更新总统计失败%+v", err)
		}
		// 更新历史统计
		today := time.Now().Truncate(24 * time.Hour)
		builder := l.svcCtx.AdminPromotionLinkStatsHistoryModel.SelectBuilder()
		builder = builder.Where("link_id = ? AND DATE(stats_date) = DATE(?)", link.Id, today)
		historyStats, err := l.svcCtx.AdminPromotionLinkStatsHistoryModel.FindAll(l.ctx, builder, "")
		if err != nil {
			return fmt.Errorf("查询今日统计记录失败%+v", err)
		}

		if len(historyStats) == 0 {
			// 创建今天的记录
			newStats := &model.AdminPromotionLinkStatsHistory{
				LinkId:     link.Id,
				StatsDate:  today,
				ClickCount: 1,
			}
			_, err = l.svcCtx.AdminPromotionLinkStatsHistoryModel.Insert(l.ctx, session, newStats)
			if err != nil {
				return fmt.Errorf("创建今日统计记录失败%+v", err)
			}
		} else {
			// 更新今日记录
			historyStats[0].ClickCount++
			err = l.svcCtx.AdminPromotionLinkStatsHistoryModel.UpdateWithVersion(l.ctx, session, historyStats[0])
			if err != nil {
				return fmt.Errorf("更新历史统计失败%+v", err)
			}
		}
		return nil
	})
	if err != nil {
		return nil, errors.Wrapf(xerr.NewErrCode(xerr.DB_ERROR), "更新总统计失败，%+v", err)
	}
	return &types.RecordLinkClickResp{
		Success: true,
	}, nil
}
